#version 430
#define NUMOFLIGHTS 3

uniform float offset[5] = float[]( 0.0, 1.0, 2.0, 3.0, 4.0 );
uniform float weight[5] = float[]( 0.2270270270, 0.1945945946, 0.1216216216,
                                   0.0540540541, 0.0162162162 );


layout (location=1) uniform sampler2D colorTexture;
layout (location=2) uniform sampler2DShadow shadowMap;
layout (location=3) uniform sampler2D normalTexture;
layout (location=4) uniform sampler2D bloomTexture;
layout (location=5) uniform sampler2D glowMap;
layout (location=6) uniform sampler2D startScreen;
//layout (location=7) uniform vec4 lightvec;

uniform int isGlowing; //0 = Object should not glow, 1 = Object should glow
uniform int pass; //0 = Shadow, 1 = NormalMapped BlinPhong, 2 = BloomDraw (no lighting)

uniform int ViewPortWidth;
uniform int ViewPortHeight;
uniform int nonormals;
uniform vec4 lights[NUMOFLIGHTS];
//uniform vec3 attenuation;

in vec3 P; /** Position **/
in vec3 N; /** Normals Worldspace**/
in vec3 V; /** eyeDirection **/
in vec3 T; /** Tangents **/
in vec3 B; /** Bitangents **/
in vec2 texCoord; /** uvs **/
in vec4 gl_FragCoord;
in vec3 viewPos;
in vec4 smCoords;

float shininess = 100.0;
vec3 L, Il, Vn, Ln;

out float gl_FragDepth;
layout (location = 0) out vec4 finalColor;

void main() {
	if(pass == 0){
	}else if(pass == 1){
		float shadowMult = textureProj(shadowMap, smCoords).r*0.8f+0.2f;
		float kc = 1.0;
		float kl = 0.014;
		float kq = 0.0007;
		vec3 normaltex = vec3(texture2D(normalTexture, texCoord).rgb);

		for(int i = 0; i < 2; i++) {

			if(lights[i].w==1) {
				Il = vec3(0.8f,1.0f,0.7f);
				L = normalize(vec3(lights[i].xyz-P));
				float d = length(vec3(lights[i].xyz - P));
				//float kc = attenuation.x;
				//float kl = attenuation.y;
				//float kq = attenuation.z;
				float att = (kc + (kl*d) + (kq*d*d));
				Il = Il * (1/att);
			} else {
				Il = vec3(1.0f,1.0f,1.0f);
				L = normalize(lights[i].xyz);
			}

			float cosNL, diffuseFactor;
			vec3 H;

			if(nonormals == 1) {
				Il = Il*1.2;
				/** Halfway Vector**/
				H = normalize(vec3(V + L));

				cosNL = max(dot(N, L), 0.1);

				diffuseFactor = cosNL * max(dot(N, H), 0.1);
			} else {
				
				/** Bitangents**/
				vec3 B = cross(N, T);

				/** new Viewvector **/
				Vn = normalize(vec3(dot(V, T), dot(V, B), dot(V, N)));
		 
				/** new Lightvector **/
				Ln = normalize(vec3(dot(L, T), dot(L, B), dot(L, N)));

				/** Halfway Vector**/
				H = normalize(vec3(Vn + Ln));

				cosNL = max(dot(normaltex, L), 0.1);

				diffuseFactor = cosNL * max(dot(normaltex, L), 0.1);
			}

			float specularFactor = max(dot(N, H), 0.05);
			specularFactor = pow(specularFactor, shininess);
		
			vec3 diffusePart = texture2D(colorTexture, texCoord).rgb *shadowMult* diffuseFactor;
			vec3 specularPart = texture2D(colorTexture, texCoord).rgb *shadowMult* specularFactor;

			//vec3 diffusePart = vec3(0.714, 0.302, 0.224) * diffuseFactor;
			//vec3 specularPart = vec3(0.714, 0.302, 0.224) * specularFactor;
			vec4 bloomFrag = texture2D(bloomTexture, vec2(gl_FragCoord.x/ViewPortWidth,gl_FragCoord.y/ViewPortHeight));
			vec4 bloomFactor = vec4(0);
			if(bloomFrag != vec4(1)){
				bloomFactor = vec4(bloomFrag.rgb,bloomFrag.b);
				finalColor += vec4(Il * 2 * (diffusePart + specularPart)/2+bloomFactor.rgb/2,1.0);
			}else{
				finalColor += vec4(Il * 2 * (diffusePart + specularPart), 1.0);
			}
		}
		finalColor = vec4(finalColor.rgb,1.0);
	}else if(pass == 2){ //Draw in Bloom Buffer
		float blue = texture2D(glowMap, texCoord).b;
		float red = texture2D(glowMap, texCoord).r;
		if(blue > 0.5 && red < 0.1 && isGlowing == 1 && viewPos.z < 100){
				finalColor = vec4(0,0.0,1.0, 1.0);
		}else{
			finalColor = vec4(1,1,1,1);
		}

	}else if(pass == 3){	//Alpha Extraction

		vec4 color = texture2D( bloomTexture, vec2(texCoord) );
		if (color != vec4(1))
			finalColor = color;
		else
			finalColor = vec4(0);

	}else if(pass == 4){	//Horizontal Blur
		vec4 color = texture2D( bloomTexture, vec2(texCoord) ) * weight[0];
    for (int i=1; i<5; i++) {
        color +=
            texture2D( bloomTexture, ( vec2(texCoord)+vec2(0.0, offset[i])/ViewPortWidth ) )
                * weight[i];
        color +=
            texture2D( bloomTexture, ( vec2(texCoord)-vec2(0.0, offset[i])/ViewPortWidth ) )
                * weight[i];
    }
    if(color.b != 0)
      finalColor = vec4(0,color.g,color.b,color.a*2);
     else
      finalColor = vec4(0);
	}else if(pass == 5){	//Vertical Blur
		vec4 color = texture2D( bloomTexture, vec2(texCoord) ) * weight[0];
		for (int i=1; i<5; i++) {
			color +=
				texture2D( bloomTexture, ( vec2(texCoord)+vec2(offset[i], 0.0)/ViewPortWidth ) )
					* weight[i];
			color +=
				texture2D( bloomTexture, ( vec2(texCoord)-vec2(offset[i], 0.0)/ViewPortWidth ) )
					* weight[i];
		}
		if(color.a != 0)
		  finalColor = vec4(0,1.0,color.b,color.a);
		else
		  finalColor = vec4(0);
	}else if(pass == 6){	//Start Screen
		finalColor = vec4(texture2D(startScreen, vec2(texCoord)).rgb, 1.0);
	}else if(pass == 7){	//Transparent "Balls"
		finalColor = vec4(1.0,1.0,1.0,0.3);
	}else if(pass == 8){	//No Transparency
		finalColor = vec4(1.0,1.0,1.0,1.0);
	}

}