#version 430

const mat4 bias = mat4(
0.5, 0.0, 0.0, 0.0,
0.0, 0.5, 0.0, 0.0,
0.0, 0.0, 0.5, 0.0,
0.5, 0.5, 0.5, 1.0
);
uniform mat4 modelMatrix;
uniform mat4 projectionMatrix;
uniform mat4 viewMatrix;

uniform mat4 lightViewMatrix;
uniform mat4 lightProjectionMatrix;

uniform bool drawingInstanced;
uniform int pass; //0 = shadowPass, 1=drawPass, 2=bloom
vec3 camPosition = vec3(0,0,0);

layout (location = 0) in vec2 uv;
layout (location = 1) in vec3 normal;
layout (location = 2) in vec3 position;
layout (location = 3) in vec3 tangent;
//layout (location = 3) in vec3 bitangent;
in mat4 modelMat;

out vec3 P; /** Position **/
out vec3 N; /** Normals Worldspace**/
out vec3 V; /** eyeDirection **/
out vec3 T; /** Tangents **/
//out vec3 B; /** Bitangents **/
out vec2 texCoord; /** uvs **/
out vec4 smCoords;
out vec3 viewPos;

void main(){
	mat4 mMatrix;
	if(drawingInstanced){
		mMatrix = modelMat;
	}else{
		mMatrix = modelMatrix;
	}

	if(pass == 0){
		gl_Position = lightProjectionMatrix * lightViewMatrix * mMatrix * vec4(position,1);
	}else if(pass == 1 || pass == 2 || pass == 7 || pass == 8){
		mat3 NormalMatrix = transpose(mat3(inverse(mMatrix)));
		texCoord = uv;
		P = (mMatrix * vec4(position, 1.0)).xyz;
		N = normalize(vec3(NormalMatrix * normal));
		V = normalize(vec3(camPosition-P));
		T = normalize(vec3(NormalMatrix * tangent));

		smCoords = bias*(lightProjectionMatrix * lightViewMatrix * mMatrix * vec4(position,1));
		gl_Position = projectionMatrix * viewMatrix * mMatrix * vec4(position,1);
		viewPos = vec3(gl_Position);
	}else if(pass == 3||pass == 4||pass == 5||pass == 6){
		texCoord = (position.xy+vec2(1,1))/2.0;
		gl_Position = vec4(position,1);
	}
}